@@ -1,5 +1,5 @@
 /* commands/dcc.cpp
-   Copyright (C) 2024 Markus Uhlin. All rights reserved.
+   Copyright (C) 2024, 2025 Markus Uhlin. All rights reserved.
 
    Redistribution and use in source and binary forms, with or without
    modification, are permitted provided that the following conditions are met:
@@ -41,6 +41,11 @@
 #include <arpa/inet.h>
 #endif
 
+#if HAVE_STD_FS
+#include <algorithm>
+#include <filesystem>
+namespace fs = std::filesystem;
+#endif
 #include <inttypes.h>
 #if WIN32
 #include <io.h>
@@ -64,6 +69,7 @@
 #include "../printtext.h"
 #include "../sig.h"
 #include "../strHand.h"
+#include "../strdup_printf.h"
 #include "../theme.h"
 #include "../tls-server.h"
 
@@ -676,6 +682,56 @@ dcc_send::set_lock(int value)
 	this->lock = value;
 }
 
+#if HAVE_STD_FS
+class disk_file {
+public:
+	std::string name;
+
+	disk_file();
+	disk_file(const char *, FileType, intmax_t, fs::perms);
+
+	FileType
+	get_type(void) const
+	{
+		return (this->type);
+	}
+
+	intmax_t
+	get_size(void) const
+	{
+		return (this->size);
+	}
+
+	fs::perms
+	get_perms(void) const
+	{
+		return (this->perms);
+	}
+
+private:
+	FileType	type;
+	intmax_t	size;
+	fs::perms	perms;
+};
+
+disk_file::disk_file()
+    : type(TYPE_unknown)
+    , size(0)
+    , perms(fs::perms::none)
+{
+	this->name.assign("");
+}
+
+disk_file::disk_file(const char *p_name, FileType p_type, intmax_t p_size,
+    fs::perms p_perms)
+    : type(p_type)
+    , size(p_size)
+    , perms(p_perms)
+{
+	this->name.assign(p_name);
+}
+#endif // HAVE_STD_FS
+
 /****************************************************************
 *                                                               *
 *  -------------- Objects with external linkage --------------  *
@@ -704,7 +760,7 @@ static std::vector<dcc_send>	send_db;
 *                                                               *
 ****************************************************************/
 
-static double
+double
 percentage(double part, double total)
 {
 	return (part / total) * 100.0;
@@ -780,6 +836,8 @@ subcmd_ok(const char *subcmd)
 		return true;
 	else if (strings_match(subcmd, "list"))
 		return true;
+	else if (strings_match(subcmd, "ls"))
+		return true;
 	else if (strings_match(subcmd, "send"))
 		return true;
 	return false;
@@ -1002,6 +1060,222 @@ subcmd_list(const char *what)
 	}
 }
 
+#if HAVE_STD_FS
+static FileType
+get_file_type(const fs::directory_entry &dir_ent)
+{
+	FileType out = TYPE_unknown;
+
+	if (dir_ent.is_symlink())
+		out = TYPE_symlink;
+	else if (dir_ent.is_block_file())
+		out = TYPE_block_file;
+	else if (dir_ent.is_character_file())
+		out = TYPE_character_file;
+	else if (dir_ent.is_directory())
+		out = TYPE_directory;
+	else if (dir_ent.is_fifo())
+		out = TYPE_fifo;
+	else if (dir_ent.is_regular_file())
+		out = TYPE_regular_file;
+	else if (dir_ent.is_socket())
+		out = TYPE_socket;
+	else if (dir_ent.is_other())
+		out = TYPE_other;
+	else
+		out = TYPE_unknown;
+
+	return out;
+}
+
+static std::vector<disk_file>
+get_file_list(const char *dir)
+{
+	fs::path		path = dir;
+	fs::directory_iterator	dir_it(path);
+	std::vector<disk_file>	df_vec;
+
+	for (const fs::directory_entry &dir_ent : dir_it) {
+		FileType	 type;
+		char		*name;
+		const char	*cp;
+		fs::perms	 perms;
+		intmax_t	 size;
+
+		if (!dir_ent.exists())
+			continue;
+
+		name = sw_strdup(dir_ent.path().string().c_str());
+
+		if ((cp = strrchr(name, SLASH_CHAR)) == nullptr)
+			cp = name;
+		else
+			cp++;
+
+		if ((type = get_file_type(dir_ent)) == TYPE_regular_file)
+			size = dir_ent.file_size();
+		else
+			size = 0;
+
+		perms = dir_ent.status().permissions();
+
+		disk_file file(cp, type, size, perms);
+#if defined(__cplusplus) && __cplusplus >= 201103L
+		df_vec.emplace_back(file);
+#else
+		df_vec.push_back(file);
+#endif
+		free(name);
+	}
+
+	return (df_vec);
+}
+
+static bool
+file_list_cmp(const disk_file &obj1, const disk_file &obj2)
+{
+	const std::string	name1(obj1.name);
+	const std::string	name2(obj2.name);
+
+	for (size_t i = 0; i < name1.length() && i < name2.length(); i++) {
+		int c1, c2;
+
+		c1 = sw_isupper(name1[i]) ? tolower(name1[i]) : name1[i];
+		c2 = sw_isupper(name2[i]) ? tolower(name2[i]) : name2[i];
+		if (c1 < c2)
+			return true;
+		else if (c1 > c2)
+			return false;
+	}
+
+	return (name1.length() < name2.length() ? true : false);
+}
+
+static bool
+is_exec(fs::perms perms)
+{
+#if defined(UNIX)
+#define PCHK(_perms, _x) (((_perms) & (_x)) != fs::perms::none)
+	return (PCHK(perms, fs::perms::owner_exec) ||
+		PCHK(perms, fs::perms::group_exec) ||
+		PCHK(perms, fs::perms::others_exec));
+#elif defined(WIN32)
+	UNUSED_PARAM(perms);
+	return false;
+#endif
+}
+#endif // HAVE_STD_FS
+
+void
+list_dir(const char *dir)
+{
+#if HAVE_STD_FS
+	static const std::string	ext1(".EXE");
+	static const std::string	ext2(".exe");
+	std::vector<disk_file>		df_vec;
+
+	if (!is_directory(dir))
+		return;
+
+	try {
+		df_vec = get_file_list(dir);
+		std::sort(df_vec.begin(), df_vec.end(), file_list_cmp);
+	} catch (const std::exception &e) {
+		printtext_print("err", "%s: %s", __func__, e.what());
+		return;
+	}
+
+	printtext_print("none", "--- BEGIN: %s ---", dir);
+
+	for (disk_file &df : df_vec) {
+		char *str = nullptr;
+
+		switch (df.get_type()) {
+		case TYPE_symlink:
+			str = strdup_printf("%s%s%s%s",
+			    COLOR_symlink, df.name.c_str(), TXT_NORMAL,
+			    SYM_symlink);
+			break;
+
+		case TYPE_block_file:
+		case TYPE_character_file:
+			str = strdup_printf("%s%s%s",
+			    COLOR_device, df.name.c_str(), TXT_NORMAL);
+			break;
+		case TYPE_directory:
+			str = strdup_printf("%s%s%s%s",
+			    COLOR_directory, df.name.c_str(), TXT_NORMAL,
+			    SLASH);
+			break;
+		case TYPE_fifo:
+			str = strdup_printf("%s%s%s%s",
+			    COLOR_fifo, df.name.c_str(), TXT_NORMAL,
+			    SYM_fifo);
+			break;
+		case TYPE_regular_file:
+			if (is_exec(df.get_perms()) ||
+			    std::equal(ext1.rbegin(), ext1.rend(),
+			    df.name.rbegin()) ||
+			    std::equal(ext2.rbegin(), ext2.rend(),
+			    df.name.rbegin())) {
+				str = strdup_printf("%s%s%s%s",
+				    COLOR_exec, df.name.c_str(), TXT_NORMAL,
+				    SYM_exec);
+			} else {
+				double	size = 0.0;
+				char	unit = 'B';
+
+				dcc::get_file_size(df.get_size(), size, unit);
+				str = strdup_printf("%s %s%.1f%c%s",
+				    df.name.c_str(),
+				    LEFT_BRKT, size, unit, RIGHT_BRKT);
+			}
+			break;
+		case TYPE_socket:
+			str = strdup_printf("%s%s%s%s",
+			    COLOR_socket, df.name.c_str(), TXT_NORMAL,
+			    SYM_socket);
+			break;
+
+		case TYPE_other:
+			str = strdup_printf("%s %s%s%s", df.name.c_str(),
+			    LEFT_BRKT, "other", RIGHT_BRKT);
+			break;
+		case TYPE_unknown:
+		default:
+			str = strdup_printf("%s %s%s%s", df.name.c_str(),
+			    LEFT_BRKT, "unknown", RIGHT_BRKT);
+			break;
+		}
+
+		if (str) {
+			printtext_print("sp2", "%s", str);
+			free(str);
+		}
+	} // for
+
+	printtext_print("none", "--- END: %s ---", dir);
+#else
+#pragma message("No C++ standard filesystem")
+	UNUSED_PARAM(dir);
+	printtext_print("err", "operation not supported");
+#endif
+}
+
+static void
+subcmd_ls(const char *what)
+{
+	if (what == nullptr || strings_match(what, "")) {
+		printtext_print("err", "insufficient args");
+	} else if (strings_match(what, "up")) {
+		list_dir(dcc::get_upload_dir());
+	} else if (strings_match(what, "down")) {
+		list_dir(g_dcc_download_dir);
+	} else {
+		printtext_print("err", "what? uploads or downloads?");
+	}
+}
+
 static void
 subcmd_send(const char *nick, const char *file)
 {
@@ -1090,7 +1364,13 @@ subcmd_send(const char *nick, const char *file)
 }
 
 /*
- * usage: /dcc [clear|get|list|send] [args]
+ * usage:
+ *     /dcc [clear|get|list|ls|send] [args]
+ *     /dcc clear [get|send|completed|all]
+ *     /dcc get <nick> <file>
+ *     /dcc list [get|send|all]
+ *     /dcc ls [up|down]
+ *     /dcc send <nick> <file>
  */
 void
 cmd_dcc(const char *data)
@@ -1127,6 +1407,8 @@ cmd_dcc(const char *data)
 		subcmd_get(arg1, arg2);
 	else if (strings_match(subcmd, "list"))
 		subcmd_list(arg1);
+	else if (strings_match(subcmd, "ls"))
+		subcmd_ls(arg1);
 	else if (strings_match(subcmd, "send"))
 		subcmd_send(arg1, arg2);
 	else
@@ -1284,7 +1566,7 @@ dcc::add_file(const char *nick, const char *user, const char *host,
 void
 dcc::get_file_size(const intmax_t bytes, double &size, char &unit)
 {
-	if (bytes <= 0 || bytes > DCC_FILE_MAX_SIZE) {
+	if (bytes <= 0) {
 		size = 0.0;
 		unit = 'B';
 	} else if (bytes >= g_one_gig) {
