@@ -82,11 +82,28 @@ private:
 };
 #endif
 
+typedef enum {
+	TYPE_symlink,
+
+	TYPE_block_file,
+	TYPE_character_file,
+	TYPE_directory,
+	TYPE_fifo,
+	TYPE_regular_file,
+	TYPE_socket,
+
+	TYPE_other,
+	TYPE_unknown
+} FileType;
+
 __SWIRC_BEGIN_DECLS
 extern const int	g_one_kilo;
 extern const int	g_one_meg;
 extern const int	g_one_gig;
 
+double percentage(double part, double total);
+void list_dir(const char *);
+
 void cmd_dcc(const char *);
 
 void dcc_init(void);
