@@ -1,5 +1,5 @@
 /* Handles event PRIVMSG
-   Copyright (C) 2016-2024 Markus Uhlin. All rights reserved.
+   Copyright (C) 2016-2025 Markus Uhlin. All rights reserved.
 
    Redistribution and use in source and binary forms, with or without
    modification, are permitted provided that the following conditions are met:
@@ -44,6 +44,7 @@
 #include "../printtext.h"
 #include "../strHand.h"
 #include "../strdup_printf.h"
+#include "../terminal.h"
 #include "../theme.h"
 
 #if defined(WIN32) && defined(TOAST_NOTIFICATIONS)
@@ -88,8 +89,8 @@ acknowledge_ctcp_request(CSTRING cmd, const struct special_msg_context *ctx)
 static void
 handle_special_msg(const struct special_msg_context *ctx)
 {
-	PRINTTEXT_CONTEXT ptext_ctx;
-	STRING msg = sw_strdup(ctx->msg);
+	PRINTTEXT_CONTEXT	ptext_ctx;
+	STRING			msg = sw_strdup(ctx->msg);
 
 	printtext_context_init(&ptext_ctx, NULL, TYPE_SPEC_NONE, true);
 	squeeze(msg, "\x01");
@@ -123,6 +124,17 @@ handle_special_msg(const struct special_msg_context *ctx)
 		    g_ascii_soh) < 0)
 			(void) atomic_swap_bool(&g_connection_lost, true);
 		acknowledge_ctcp_request("TIME", ctx);
+	} else if (!strncmp(msg, "USERINFO", 9) &&
+		   config_bool("ctcp_reply", true)) {
+		CSTRING ui_str = Config("ctcp_userinfo");
+
+		if (net_send("NOTICE %s :%cUSERINFO %s%c",
+		    ctx->nick,
+		    g_ascii_soh,
+		    (strings_match(ui_str, "") ? "No info set" : ui_str),
+		    g_ascii_soh) < 0)
+			(void) atomic_swap_bool(&g_connection_lost, true);
+		acknowledge_ctcp_request("USERINFO", ctx);
 	} else if (!strncmp(msg, "VERSION", 8) &&
 		   config_bool("ctcp_reply", true)) {
 		if (net_send("NOTICE %s :%cVERSION Swirc %s by %s  --  %s%c",
@@ -150,6 +162,7 @@ broadcast_window_activity(const IRC_WINDOW *src)
 		    TYPE_SPEC1_SUCCESS, true);
 		printtext(&ctx, "activity at window %c%s%c (refnum: %d)",
 		    BOLD, src->label, BOLD, src->refnum);
+		term_beep();
 	}
 }
 
