@@ -1,5 +1,5 @@
 /* main.cpp  --  starts execution
-   Copyright (C) 2012-2024 Markus Uhlin. All rights reserved.
+   Copyright (C) 2012-2025 Markus Uhlin. All rights reserved.
 
    Redistribution and use in source and binary forms, with or without
    modification, are permitted provided that the following conditions are met:
@@ -74,6 +74,7 @@
 #include "window.h"
 
 #include "commands/dcc.h"
+#include "commands/ftp.h"
 
 #if defined(WIN32) && defined(TOAST_NOTIFICATIONS)
 #include "DesktopNotificationManagerCompat.hpp"
@@ -87,8 +88,8 @@
 /* Things with external linkage
    ============================ */
 
-chararray_t	g_swircVersion	= "v3.5.1";
-chararray_t	g_swircYear	= "2012-2024";
+chararray_t	g_swircVersion	= "v3.5.2";
+chararray_t	g_swircYear	= "2012-2025";
 chararray_t	g_swircAuthor	= "Markus Uhlin";
 chararray_t	g_swircWebAddr	= "https://www.nifty-networks.net/swirc/";
 
@@ -549,12 +550,6 @@ main(int argc, char *argv[])
 	malloc_options = const_cast<char *>("S");
 #endif
 
-#if defined(UNIX)
-#define SLASH_CHAR '/'
-#elif defined(WIN32)
-#define SLASH_CHAR '\\'
-#endif
-
 	if ((cp = strrchr(argv[0], SLASH_CHAR)) == nullptr)
 		g_progname = argv[0];
 	else
@@ -689,6 +684,7 @@ main(int argc, char *argv[])
 	windowSystem_init();
 	readline_init();
 	net_ssl_init();
+	ftp_init();
 
 #if defined(UNIX) && defined(NDEBUG)
 	struct rlimit rlim = { 0 };
@@ -743,6 +739,12 @@ main(int argc, char *argv[])
 			return EXIT_FAILURE;
 		}
 	}
+	if (ftp::want_unveil_uploads()) {
+		if (unveil(ftp::get_upload_dir(), "r") == -1) {
+			err_ret("unveil");
+			return EXIT_FAILURE;
+		}
+	}
 #endif
 
 #if defined(OpenBSD) && OpenBSD >= 201605
@@ -766,6 +768,7 @@ main(int argc, char *argv[])
 	 * Reverse order...
 	 */
 	dcc_deinit();
+	ftp_deinit();
 	net_ssl_deinit();
 #ifdef WIN32
 	winsock_deinit();
